@@ -12,10 +12,10 @@ EXAMPLE 3: Vortex with inductance: Flux quantization
 
 if __name__ == "__main__":
     # Square array with screening
-    N = 14
+    N = 100
     array = SquareArray(N, N)
 
-    Llist = [0.001, 0.1, 1, 5]
+    Llist = [0.001, 0.001, 0.1, 1, 5]
 
     f = 1 / (N - 1) ** 2  # frustration
     n = np.zeros(array.face_count(), dtype=int)  # target vortex configuration
@@ -25,11 +25,11 @@ if __name__ == "__main__":
         array.set_inductance_factors(L)
         prob = StaticProblem(array, frustration=f, vortex_configuration=n)
         out, _, info = prob.compute()
-        out.plot(face_quantity="flux", face_quantity_logarithmic_colors=True,
-                 face_quantity_clim=[1E-3, 1], title=f"beta_L={L}", arrow_color=[1, 1, 1])
+        # out.plot(face_quantity="flux", face_quantity_logarithmic_colors=True,
+        #          face_quantity_clim=[1E-3, 1], title=f"beta_L={L}", arrow_color=[1, 1, 0.3])
+        print(info)
 
-
-    plt.show()
+    # plt.show()
 
 
 
