@@ -596,7 +596,7 @@ class Circuit:
         junc_L : scalar float
             Self-inductance prefactor.
         junc_M : scalar float
-            Mutual-inductance prefactor.
+            Mutual-inductance prefactor (usually << junc_L).
         max_dist : scalar float
             Cut-off distance between junctions included in L.
 
@@ -780,11 +780,12 @@ class Circuit:
 
     @staticmethod
     def _prepare_inducance_matrix(L, N):
+        eps = 10 * np.finfo(float).eps
         if not hasattr(L, "ndim"):
             L = np.array(L)
         if L.ndim <= 1:
             x = Circuit._prepare_junction_quantity(L, N, "L")
-            return scipy.sparse.diags(x, 0).tocsc(), np.all(x >= 0), np.any(x != 0.0)
+            return scipy.sparse.diags(x, 0).tocsc(), np.all(x > -eps), np.any(x != 0.0)
         if L.shape == (N, N):
             if not Circuit._is_symmetric(L):
                 raise ValueError("inductance matrix must be symmetric")
@@ -852,7 +853,7 @@ class Circuit:
 
     def _has_capacitance(self):
         # returns False if self.capacitance_factors is zero, True otherwise
-        np.any(self.capacitance_factors > 0)
+        return np.any(self.capacitance_factors > 0)
 
     def _has_inductance(self):
         # returns False if self.inductance_factors is zero, True otherwise
